/*
    Génération de feuille de personnage complète AD&D 1 et 2
    Génération aléatoire de personnage

    Copyright (C) 2011 Bernard SIAUD

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <dirent.h>
#ifdef __linux__
#include <unique/unique.h>


enum
{
  COMMAND_0, /* unused: 0 is an invalid command */

  COMMAND_FOO,
  COMMAND_BAR
};
static UniqueResponse attente (UniqueApp * app,UniqueCommand command,UniqueMessageData *message,guint time_,gpointer param);
#endif

#include "include.h"


void ouverture      (GtkWidget *wid, gpointer param);
void nouveau        (GtkWidget *wid, gpointer param);

int main (int argc, char *argv[])
{
  GtkWidget *button = NULL;
  GtkWidget *win = NULL;
  GtkWidget *vbox = NULL;
  GtkAccelGroup *pAccel;
  GError * error;
  void ** param; /* pour le passage des paramètres */
  /* Definition des elements du menu */
   /* 0: fenêtre */
   /* 1: préférence des caractéristiques => mémoriser le dernier état */
   /* 2: menubar    */
   /* 3 : GtkActionGroup */
   /* 4 : xml */
   /* 5 : chemin_socket */
   /* 6 : lui-même */
   /* 7 : gtkentry pour fils */
   /* 8 : list */
   /* 9 : sd2 */
   /* 10 : sd */
  GtkUIManager *menu_Manager;
  GtkWidget *menubar;
  /* Le menu complet, sans
  - les boutons radio, tout de même l'entrée du menu relative aux boutons radios : ModeDeTirage
  - les cases à cocher
  - les séparateurs (qui n'auront besoin d'aucune spécification nulle part */
  GtkActionEntry entrees_menu[] =
  {
   {"FichierMenuAction",NULL           ,"_Jeu"        , NULL       , NULL                   , NULL                        },
   {"Tirer"            ,"_3D6"         ,"_Tirer"      ,"<Control>T","Tirage d'un personnage", G_CALLBACK (tirage         )},
   {"hist"             ,GTK_STOCK_PRINT_PREVIEW,"_Historique", NULL, NULL                   , NULL                        },
   {"Quitter"          ,GTK_STOCK_QUIT ,"_Quitter"    ,"<Control>Q","Finir la partie"       , G_CALLBACK( cb_fin         )},
   {"vdd"              ,NULL           ,"_Version du jeu", NULL    , NULL                   , NULL                        },
   {"AideMenuAction"   ,NULL           ,"_Aide"       , NULL       , NULL                   , NULL                        },
   {"Aide"             ,GTK_STOCK_HELP ,"_Aide"       ,"<Shift>A"  , NULL                   , G_CALLBACK( menu_aide      )},
   {"AproposAction"    ,GTK_STOCK_ABOUT,"A _propos"   ,NULL        , NULL                   , G_CALLBACK( menu_a_propos  )},
   {"Licence"          ,GTK_STOCK_INFO ,"_Licence GPL","<Release>L", NULL                   , G_CALLBACK( menu_licence   )},
   {"Vue"              ,GTK_STOCK_ABOUT,"Édition _Feuilles",NULL,NULL        , NULL                        },
   {"Race"             ,GTK_STOCK_ABOUT,"_Races"       , NULL      , NULL                   , NULL                        },
   {"Classe"           ,GTK_STOCK_ABOUT,"_Classes"     , NULL      , NULL                   , NULL                        },
   {"Tirage"           ,"_3D6x6"       ,"Choix _Tirage" , NULL,NULL     , NULL                        },
   {"ModeDeTirage"     ,"_5D6"         ,"_Mode de tirage",NULL     , NULL                   , NULL                        },
   {"Classes"          ,"_4D5p3"       ,"_Classes"     , NULL      , NULL                   , NULL                        },
   {"race"             ,NULL           ,"_Race"        , NULL      , NULL                   , NULL                        },
   {"sexe"             ,NULL           ,"Se_Xe"        , NULL      , NULL                   , NULL                        },
   {"preference"       ,NULL           ,"_Préférence"  , NULL      , NULL                   , NULL                        },
   {"force"            ,NULL           ,"_Force : 1"   , NULL      , NULL                   , NULL                        },
   {"intelligence"     ,NULL           ,"_Intelligence : 2", NULL  , NULL                   , NULL                        },
   {"sagesse"          ,NULL           ,"_Sagesse : 3" , NULL      , NULL                   , NULL                        },
   {"dexterite"        ,NULL           ,"_Dexterité : 4", NULL     , NULL                   , NULL                        },
   {"constitution"     ,NULL           ,"C_onstitution : 5",NULL   , NULL                   , NULL                        },
   {"charisme"         ,NULL           ,"_Charisme : 6" ,NULL      , NULL                   , NULL                        },
   {"beaute"           ,NULL           ,"_Beauté : X"   , NULL     , NULL                   , NULL                        },
   {"humour"           ,NULL           ,"_Humour : X"   , NULL     , NULL                   , NULL                        },
   {"pdv"              ,"_2D8"         ,"_Points de vie",NULL      , NULL                   , NULL                        },
   {"niveau"           ,"_2D8"         ,"_Niveau"       ,NULL      , NULL                   , NULL                        }
  };
  /* Radio items */
 static const GtkRadioActionEntry v_radio_entries[] =
 {
   { "add1", "ADD1", "Avanced Donjon & Dragon"   , NULL, "Je joue à Avanced Dongeon et Dragon"                         , ADD1 },
   { "add2", "ADD2", "Avanced Donjon & Dragon V2", NULL, "Je joue à Avanced Dongeon et Dragon V2"                      , ADD2 }
 };
 static const GtkRadioActionEntry x_radio_entries[] =
 {
   { "homme", "HOMME", "Masculin", NULL, "Le personne est masculin",HOMME},
   { "femme", "FEMME", "Féminin" , NULL, "Le personne est feminin" ,FEMME}
 };
 static const GtkRadioActionEntry radio_entries[] =
 {
   { "_3D6"  , "_3D6" , "_3 dés à 6 faces", NULL, "Tirages des caractéristiques avec 3 dès à 6 faces", _3D6 },
   { "_4D6"  , "_4D6"  , "_4 dés à 6 faces, 3 meilleurs", NULL, "Tirages des caractéristiques avec 4 dés à 6 faces, somme des 3 meilleurs", _4D6 },
   { "_5D6"  , "_5D6"  , "_5 dés à 6 faces, 3 meilleurs", NULL, "Tirages des caractéristiques avec 5 dés à 6 faces, somme des 3 meilleurs", _5D6 },
   { "_3D6x6", "_3D6x6", "_6 fois 3 dés à 6 faces, le meilleur", NULL, "Tirages des caractéristiques avec 6 fois 3 dés à 6 faces, on garde le meilleur", _3D6x6 },
   { "_4D5p3", "_4D5p3", "4 dés à 5 faces, 3 meilleurs _+ 3", NULL, "Tirages des caractéristiques avec 4 dés à 5 faces, somme des 3 meilleurs plus 3", _4D5p3 },
 };
 static const GtkRadioActionEntry f_radio_entries[] =
 {
   { "f1"  , NULL , "_1", NULL, "1", 1 },
   { "f2"  , NULL , "_2", NULL, "2", 2 },
   { "f3"  , NULL , "_3", NULL, "3", 3 },
   { "f4"  , NULL , "_4", NULL, "4", 4 },
   { "f5"  , NULL , "_5", NULL, "5", 5 },
   { "f6"  , NULL , "_6", NULL, "6", 6 },
   { "f7"  , NULL , "_7", NULL, "7", 7 },
   { "f8"  , NULL , "_8", NULL, "8", 8 }
 };
 static const GtkRadioActionEntry i_radio_entries[] =
 {
   { "i1"  , NULL , "1", NULL, "1", 1 },
   { "i2"  , NULL , "2", NULL, "2", 2 },
   { "i3"  , NULL , "3", NULL, "3", 3 },
   { "i4"  , NULL , "4", NULL, "4", 4 },
   { "i5"  , NULL , "5", NULL, "5", 5 },
   { "i6"  , NULL , "6", NULL, "6", 6 },
   { "i7"  , NULL , "7", NULL, "7", 7 },
   { "i8"  , NULL , "8", NULL, "8", 8 }
 };
 static const GtkRadioActionEntry s_radio_entries[] =
 {
   { "s1"  , NULL , "1", NULL, "1", 1 },
   { "s2"  , NULL , "2", NULL, "2", 2 },
   { "s3"  , NULL , "3", NULL, "3", 3 },
   { "s4"  , NULL , "4", NULL, "4", 4 },
   { "s5"  , NULL , "5", NULL, "5", 5 },
   { "s6"  , NULL , "6", NULL, "6", 6 },
   { "s7"  , NULL , "7", NULL, "7", 7 },
   { "s8"  , NULL , "8", NULL, "8", 8 }
 };
 static const GtkRadioActionEntry d_radio_entries[] =
 {
   { "d1"  , NULL , "1", NULL, "1", 1 },
   { "d2"  , NULL , "2", NULL, "2", 2 },
   { "d3"  , NULL , "3", NULL, "3", 3 },
   { "d4"  , NULL , "4", NULL, "4", 4 },
   { "d5"  , NULL , "5", NULL, "5", 5 },
   { "d6"  , NULL , "6", NULL, "6", 6 },
   { "d7"  , NULL , "7", NULL, "7", 7 },
   { "d8"  , NULL , "8", NULL, "8", 8 }
 };
 static const GtkRadioActionEntry co_radio_entries[] =
 {
   { "co1"  , NULL , "1", NULL, "1", 1 },
   { "co2"  , NULL , "2", NULL, "2", 2 },
   { "co3"  , NULL , "3", NULL, "3", 3 },
   { "co4"  , NULL , "4", NULL, "4", 4 },
   { "co5"  , NULL , "5", NULL, "5", 5 },
   { "co6"  , NULL , "6", NULL, "6", 6 },
   { "co7"  , NULL , "7", NULL, "7", 7 },
   { "co8"  , NULL , "8", NULL, "8", 8 }
 };
 static const GtkRadioActionEntry ch_radio_entries[] =
 {
   { "ch1"  , NULL , "1", NULL, "1", 1 },
   { "ch2"  , NULL , "2", NULL, "2", 2 },
   { "ch3"  , NULL , "3", NULL, "3", 3 },
   { "ch4"  , NULL , "4", NULL, "4", 4 },
   { "ch5"  , NULL , "5", NULL, "5", 5 },
   { "ch6"  , NULL , "6", NULL, "6", 6 },
   { "ch7"  , NULL , "7", NULL, "7", 7 },
   { "ch8"  , NULL , "8", NULL, "8", 8 }
 };
 static const GtkRadioActionEntry b_radio_entries[] =
 {
   { "b1"  , NULL , "1", NULL, "1", 1 },
   { "b2"  , NULL , "2", NULL, "2", 2 },
   { "b3"  , NULL , "3", NULL, "3", 3 },
   { "b4"  , NULL , "4", NULL, "4", 4 },
   { "b5"  , NULL , "5", NULL, "5", 5 },
   { "b6"  , NULL , "6", NULL, "6", 6 },
   { "b7"  , NULL , "7", NULL, "7", 7 },
   { "b8"  , NULL , "8", NULL, "8", 8 },
   { "bim" , NULL , "X", NULL, "X", 9 }
 };
 static const GtkRadioActionEntry h_radio_entries[] =
 {
   { "h1"  , NULL , "1", NULL, "1", 1 },
   { "h2"  , NULL , "2", NULL, "2", 2 },
   { "h3"  , NULL , "3", NULL, "3", 3 },
   { "h4"  , NULL , "4", NULL, "4", 4 },
   { "h5"  , NULL , "5", NULL, "5", 5 },
   { "h6"  , NULL , "6", NULL, "6", 6 },
   { "h7"  , NULL , "7", NULL, "7", 7 },
   { "h8"  , NULL , "8", NULL, "8", 8 },
   { "him" , NULL , "X", NULL, "X", 9 }
 };
 static const GtkRadioActionEntry pdv_radio_entries[] =
 {
   { "pdv_1d"  , "_1D8" , "1 dé", NULL, "Un seul dé", 1 },
   { "pdv_2d"  , "_2D8" , "2 dés", NULL, "Le meilleur de deux dés", 2 },
   { "pdv_max" , NULL , "Maximum", NULL, "Le maximum", 3 }
 };
 static const GtkRadioActionEntry niv_radio_entries[] =
 {
   { "n0"  , NULL , "Niveau 0" , NULL, "Niveau 0" ,  0 },
   { "n1"  , NULL , "Niveau 1" , NULL, "Niveau 1" ,  1 },
   { "n2"  , NULL , "Niveau 2" , NULL, "Niveau 2" ,  2 },
   { "n3"  , NULL , "Niveau 3" , NULL, "Niveau 3" ,  3 },
   { "n4"  , NULL , "Niveau 4" , NULL, "Niveau 4" ,  4 },
   { "n5"  , NULL , "Niveau 5" , NULL, "Niveau 5" ,  5 },
   { "n6"  , NULL , "Niveau 6" , NULL, "Niveau 6" ,  6 },
   { "n7"  , NULL , "Niveau 7" , NULL, "Niveau 7" ,  7 },
   { "n8"  , NULL , "Niveau 8" , NULL, "Niveau 8" ,  8 },
   { "n9"  , NULL , "Niveau 9" , NULL, "Niveau 9" ,  9 },
   { "n10" , NULL , "Niveau 10", NULL, "Niveau 10", 10 },
   { "n11" , NULL , "Niveau 11", NULL, "Niveau 11", 11 },
   { "n12" , NULL , "Niveau 12", NULL, "Niveau 12", 12 },
   { "n13" , NULL , "Niveau 13", NULL, "Niveau 13", 13 },
   { "n14" , NULL , "Niveau 14", NULL, "Niveau 14", 14 },
   { "n15" , NULL , "Niveau 15", NULL, "Niveau 15", 15 },
   { "n16" , NULL , "Niveau 16", NULL, "Niveau 16", 16 },
   { "n17" , NULL , "Niveau 17", NULL, "Niveau 17", 17 },
   { "n18" , NULL , "Niveau 18", NULL, "Niveau 18", 18 },
   { "n19" , NULL , "Niveau 19", NULL, "Niveau 19", 19 },
   { "n20" , NULL , "Niveau 20", NULL, "Niveau 20", 20 }
 };

 static struct
 {
  gchar *filename;
  gchar *stock_id;
 } stock_icons[] =
 {
  { "1d8.png", "_1D8" },
  { "2d8.png", "_2D8" },
  { "3d6.png", "_3D6" },
  { "4d6.png", "_4D6" },
  { "4d5p3.png", "_4D5p3" },
  { "5d6.png", "_5D6" },
  { "3d6x6.png", "_3D6x6" }
 };
 static gint n_stock_icons = G_N_ELEMENTS (stock_icons);
 GtkIconFactory *icon_factory;
 GtkIconSet *icon_set;
 GtkIconSource *icon_source;
 unsigned short i,j,k;
 gchar * ici, * tmp_ch;
 gchar chemin[LONG];
 unsigned short val[8]={1,2,3,4,5,6,7,8};
 struct dirent * ent;
 struct_classe * classe=NULL, tmp_classe;
 unsigned short * nb_classe =(unsigned short *)malloc(sizeof(unsigned short));
 struct_race * race=NULL, tmp_race;
 unsigned short * nb_race =(unsigned short *)malloc(sizeof(unsigned short));
 struct_save * save=NULL;
 unsigned short * nb_save =(unsigned short *)malloc(sizeof(unsigned short));
 struct_sort * sort=NULL;
 unsigned short * nb_sort =(unsigned short *)malloc(sizeof(unsigned short));
 DIR * rep;
 signed short drap;
 GtkActionGroup * p_actionGroup; /*, * p_actionGroup1 */
#ifdef __linux__
 UniqueApp *app;
 UniqueMessageData * unique_message;
#endif
 /* initialisation GTK */
 gtk_init (&argc, &argv);
#ifdef __linux__
 app = unique_app_new_with_commands ("org.troumad.perso", NULL,"foo", COMMAND_FOO,"bar", COMMAND_BAR,NULL);
 if (unique_app_is_running (app))
 {
    printf("Envoie de message au programme qui tourne déjà\n");
    for(i=1;i<argc;i++)
    {
        unique_message=unique_message_data_new();
        unique_message_data_set_text(unique_message,argv[i],strlen(argv[i]));
        unique_app_send_message (app, 1, unique_message);
        unique_message_data_free(unique_message);
    }
    g_object_unref (app);
    return 0;
  }
#endif

 param=(void **)malloc(MAX_MAIN*sizeof(void *)) ; /* pour le passage des paramètres */

 ici=chemin_exe();

 sprintf(chemin,"%s%craces",ici,SEPARATEUR);
 rep = opendir(chemin);
 i=0;
 if (rep != NULL)
 {
    while ((ent = readdir(rep)) != NULL)
    { /* http://c.developpez.com/faq/?page=fichiers#FICHIERS_dir_list */
        gchar *ch,*point=NULL;


        for (ch=(gchar *)ent->d_name;*ch!=0;ch++)
        {
            if (*ch==SEPARATEUR || *ch=='\\') point=NULL;
            else if (*ch=='.') point=ch;
        }

        if (ch-point==4 && strncmp(point,".ods",4)==0)
        {
            race=(struct_race *)realloc(race,(i+1)*sizeof(struct_race));
            tmp_race.add1.nom=NULL;
            tmp_race.add2.nom=NULL;
            tmp_race.menu=NULL;
            sprintf(chemin,"%s%craces%c%s",ici,SEPARATEUR,SEPARATEUR,(char *)ent->d_name);
            retourne_race (&tmp_race,chemin);
            if (tmp_race.add1.nom==NULL)
                tmp_ch=tmp_race.add2.nom;
            else
                tmp_ch=tmp_race.add1.nom;
            if (tmp_ch!=NULL)
            {
                j=0;
                drap=1;
                while(j<i && drap)
                {
                    if (race[j].add1.nom!=NULL && strcmp(tmp_ch,race[j].add1.nom)<0)
                    {
                        drap=0;
                    }
                    else if (race[j].add2.nom!=NULL && strcmp(tmp_ch,race[j].add2.nom)<0)
                    {
                        drap=0;
                    }
                    else
                        j++;
                }
                for(k=i;k>j;k--)
                {
                    race[k]=race[k-1];
                }
                race[j]=tmp_race;
                i++;
            }
        }
    }
    *nb_race=i;
    param[MAIN_PARAM_RACE]=malloc(i*sizeof(struct_param_nb));
    while (i>0)
    {
         i--;
        ((struct_param_nb **)param)[MAIN_PARAM_RACE][i].nb=i;
        ((struct_param_nb **)param)[MAIN_PARAM_RACE][i].param=param;
    }

    closedir(rep);
 }

 param[MAIN_PARAM_HIST]=malloc(MEM_FICH_MAX*sizeof(struct_param_nb));
 for (i=0;i<MEM_FICH_MAX;i++)
 {
    ((struct_param_nb **)param)[MAIN_PARAM_HIST][i].nb=i+1;
    ((struct_param_nb **)param)[MAIN_PARAM_HIST][i].param=param;
 }

 sprintf(chemin,"%s%cfichiers",ici,SEPARATEUR);
 rep = opendir(chemin);
 i=0;j=0;k=0;
 if (rep != NULL)
 {
    while ((ent = readdir(rep)) != NULL)
    { /* http://c.developpez.com/faq/?page=fichiers#FICHIERS_dir_list */
        gchar *ch,*point=NULL;


        for (ch=(gchar *)ent->d_name;*ch!=0;ch++)
        {
            if (*ch==SEPARATEUR || *ch=='\\') point=NULL;
            else if (*ch=='.') point=ch;
        }

        if (ch-point==4 && strncmp(point,".ods",4)==0)
        {
            if (strncmp("save_",(char *)ent->d_name,5)==0)
            {
                i++;
                save=(struct_save *)realloc(save,i*sizeof(struct_save));
                sprintf(chemin,"%s%cfichiers%c%s",ici,SEPARATEUR,SEPARATEUR,(char *)ent->d_name);
                save[i-1].add1.save=NULL;
                save[i-1].add2.save=NULL;
                save[i-1].add1.nom=NULL;
                save[i-1].add2.nom=NULL;
                retourne_save (save+i-1,chemin);
            }
            else
            {
                j++;
                sort=(struct_sort *)realloc(sort,j*sizeof(struct_sort));
                sprintf(chemin,"%s%cfichiers%c%s",ici,SEPARATEUR,SEPARATEUR,(char *)ent->d_name);
                sort[j-1].type=NULL;
                sort[j-1].add1.nb=NULL;
                sort[j-1].add2.nb=NULL;
                sort[j-1].add1.nom=NULL;
                sort[j-1].add2.nom=NULL;
                retourne_sort (sort+j-1,chemin);
            }
        }
    }
    *nb_save=i;
    *nb_sort=j;

    closedir(rep);
 }

 param[MAIN_RACE]=(void *)race;
 param[MAIN_NB_RACE]=(void *)nb_race;
 param[MAIN_SAVE]=(void *)save;
 param[MAIN_NB_SAVE]=(void *)nb_save;
 param[MAIN_SORT]=(void *)sort;
 param[MAIN_NB_SORT]=(void *)nb_sort;

 sprintf(chemin,"%s%cclasses",ici,SEPARATEUR);
 rep = opendir(chemin);
 i=0;
 if (rep != NULL)
 {
    while ((ent = readdir(rep)) != NULL)
    { /* http://c.developpez.com/faq/?page=fichiers#FICHIERS_dir_list */
        gchar *ch,*point=NULL;


        for (ch=(gchar *)ent->d_name;*ch!=0;ch++)
        {
            if (*ch==SEPARATEUR || *ch=='\\') point=NULL;
            else if (*ch=='.') point=ch;
        }

        if (ch-point==4 && strncmp(point,".ods",4)==0)
        {
            classe=(struct_classe *)realloc(classe,(i+1)*sizeof(struct_classe));
            tmp_classe.add1.nom=NULL;
            tmp_classe.add2.nom=NULL;
            //tmp_classe.menu=NULL;
            sprintf(chemin,"%s%cclasses%c%s",ici,SEPARATEUR,SEPARATEUR,(char *)ent->d_name);
            retourne_classe (&tmp_classe,chemin,param);
            if (tmp_classe.add1.nom==NULL)
                tmp_ch=tmp_classe.add2.nom;
            else
                tmp_ch=tmp_classe.add1.nom;
            if (tmp_ch!=NULL)
            {
                j=0;
                drap=1;
                while(j<i && drap)
                {
                    if (classe[j].add1.nom!=NULL && strcmp(tmp_ch,classe[j].add1.nom)<0)
                    {
                        drap=0;
                    }
                    else if (classe[j].add2.nom!=NULL && strcmp(tmp_ch,classe[j].add2.nom)<0)
                    {
                        drap=0;
                    }
                    else
                        j++;
                }
                for(k=i;k>j;k--)
                {
                    classe[k]=classe[k-1];
                }
                classe[j]=tmp_classe;
                i++;
            }
        }
    }
    *nb_classe=i;
    param[MAIN_PARAM_CLAS]=malloc(i*sizeof(struct_param_nb));
    while (i>0)
    {
         i--;
        ((struct_param_nb **)param)[MAIN_PARAM_CLAS][i].nb=i;
        ((struct_param_nb **)param)[MAIN_PARAM_CLAS][i].param=param;
    }
   closedir(rep);
 }
 param[MAIN_CLASSE]=(void *)classe;
 param[MAIN_NB_CLASSE]=(void *)nb_classe;


  /* Create the main window */
  win = gtk_window_new (GTK_WINDOW_TOPLEVEL); /* déclaration d'une fenêtre complète */
  param[0]=(void *)win;


/* gestion des process */

 param[1]=(void *)val;
 param[4]=NULL;
 ((gulong **)param)[12]=(gulong *)malloc(sizeof(gulong));((gulong **)param)[12][0]=1;

  /* Le premier : mode de tirage, les 8 suivants : minima particuliers, les 8 suivants : ordre de préférence, guerrier, paladin,ranger, voleur, barde, prêtre, druide, magicien, illusionniste, autre */

  /* creation des icones */
  icon_factory = gtk_icon_factory_new ();
  for (i = 0; i < n_stock_icons; i++)
  {
      icon_set = gtk_icon_set_new ();
      icon_source = gtk_icon_source_new ();
      sprintf(chemin,"%s%c%s",ici,SEPARATEUR,stock_icons[i].filename);
      gtk_icon_source_set_filename (icon_source, chemin);
      gtk_icon_set_add_source (icon_set, icon_source);
      gtk_icon_source_free (icon_source);
      gtk_icon_factory_add (icon_factory, stock_icons[i].stock_id, icon_set);
      gtk_icon_set_unref (icon_set);
  }
  gtk_icon_factory_add_default (icon_factory);
  g_object_unref (icon_factory); /* supprimer l'objet de la mémoire */


  /* Create the main window */
  gtk_container_set_border_width (GTK_CONTAINER (win), 8);
  gtk_window_set_title (GTK_WINDOW (win), "Tirage de personnages");
  gtk_window_set_position (GTK_WINDOW (win), GTK_WIN_POS_CENTER); /* centrer la fenêtre */
  gtk_widget_realize (win);
  g_signal_connect (win, "destroy", gtk_main_quit, NULL);
  /* icone */
  sprintf(chemin,"%s%c%s",ici,SEPARATEUR,"add.png");
  if (!gtk_window_set_icon_from_file(GTK_WINDOW (win),chemin,&error))
  {                                                                  /* met une icone à la fenêtre                                   */
   traite_erreur(error);
   g_error_free(error);
  }

  /* Create a vertical box with buttons */
  vbox = gtk_vbox_new (TRUE, 6);
  gtk_container_add (GTK_CONTAINER (win), vbox);



  /* initialisation du menu */
  p_actionGroup = gtk_action_group_new( "menuActionGroup" );
  /* toutes les entrées sauf boutons radio et cases à cocher */
  gtk_action_group_add_actions( p_actionGroup, entrees_menu, G_N_ELEMENTS( entrees_menu ), (gpointer)param );
  /* boutons radio */
  gtk_action_group_add_radio_actions (p_actionGroup, v_radio_entries, G_N_ELEMENTS (v_radio_entries), 2, (GCallback)version, (gpointer) param);
  gtk_action_group_add_radio_actions (p_actionGroup, radio_entries, G_N_ELEMENTS (radio_entries), 1, NULL, NULL);
  /*gtk_action_group_add_radio_actions (p_actionGroup, r_radio_entries ,  G_N_ELEMENTS (r_radio_entries), 0, (GCallback)print_race, (gpointer) param);*/
  gtk_action_group_add_radio_actions (p_actionGroup, x_radio_entries ,  G_N_ELEMENTS (x_radio_entries), 0, NULL, NULL);
  gtk_action_group_add_radio_actions (p_actionGroup, f_radio_entries ,  G_N_ELEMENTS (f_radio_entries), 1, (GCallback)print_force, (gpointer) param);
  gtk_action_group_add_radio_actions (p_actionGroup, i_radio_entries ,  G_N_ELEMENTS (i_radio_entries), 2, (GCallback)print_intelligence, (gpointer) param);
  gtk_action_group_add_radio_actions (p_actionGroup, s_radio_entries ,  G_N_ELEMENTS (s_radio_entries), 3, (GCallback)print_sagesse, (gpointer) param);
  gtk_action_group_add_radio_actions (p_actionGroup, d_radio_entries ,  G_N_ELEMENTS (d_radio_entries), 4, (GCallback)print_dexterite, (gpointer) param);
  gtk_action_group_add_radio_actions (p_actionGroup, co_radio_entries, G_N_ELEMENTS (co_radio_entries), 5, (GCallback)print_constitution, (gpointer) param);
  gtk_action_group_add_radio_actions (p_actionGroup, ch_radio_entries, G_N_ELEMENTS (ch_radio_entries), 6, (GCallback)print_charisme, (gpointer) param);
  gtk_action_group_add_radio_actions (p_actionGroup, b_radio_entries ,  G_N_ELEMENTS (b_radio_entries), 9, (GCallback)print_beaute, (gpointer) param);
  gtk_action_group_add_radio_actions (p_actionGroup, h_radio_entries ,  G_N_ELEMENTS (h_radio_entries), 9, (GCallback)print_humour, (gpointer) param);
  gtk_action_group_add_radio_actions (p_actionGroup,pdv_radio_entries, G_N_ELEMENTS(pdv_radio_entries), 1, NULL, NULL);
  gtk_action_group_add_radio_actions (p_actionGroup,niv_radio_entries, G_N_ELEMENTS(niv_radio_entries), 1, NULL, NULL);
  /* cases à cocher */
  //gtk_action_group_add_toggle_actions (p_actionGroup, toggle_entries, G_N_ELEMENTS (toggle_entries), (gpointer)param);
  /* déclaration de l'item menu */
  menu_Manager = gtk_ui_manager_new ();
  /******************************************************/
  /*  manipulation pour activer les raccourcis clavier  */
  pAccel = gtk_ui_manager_get_accel_group (menu_Manager);  /* Création de l'AccelGroup à partir du GtkActionEntry          */
  gtk_window_add_accel_group (GTK_WINDOW (win), pAccel);   /* Activation des raccourcis clavier à partir de l'AccelGroup   */
  /******************************************************/
  sprintf(chemin,"%s%cmain.menu.xml",ici,SEPARATEUR);
  gtk_ui_manager_add_ui_from_file(menu_Manager,chemin,NULL); /* création des items du menu à partir du fichier xml */
  gtk_ui_manager_insert_action_group( menu_Manager, p_actionGroup, 0 ); /* lien entre le menu est actions définies précédement */
  //gtk_ui_manager_set_add_tearoffs     (menu_Manager,TRUE); /* possibilité de décrocher les sous-menu */

  menubar = gtk_ui_manager_get_widget (menu_Manager,"/menubar"); /* pour trouver le menu */
  gtk_box_pack_start (GTK_BOX (vbox), menubar, FALSE, FALSE, 0); /* pour mettre le menu dans la fenêtre */
  param[MAIN_MENU_MAN]=(void *)menu_Manager;
  param[MAIN_GR_ACTION]=(void *)p_actionGroup;

  /* les trois boutons */
 /* button = gtk_button_new_from_stock (GTK_STOCK_DIALOG_INFO);
  g_signal_connect (G_OBJECT (button), "clicked", G_CALLBACK (explication), (gpointer) param);
  gtk_box_pack_start (GTK_BOX (vbox), button, TRUE, TRUE, 0);*/

  button = gtk_button_new_from_stock (GTK_STOCK_OPEN); /*GTK_STOCK_DIALOG_INFO);*/
  g_signal_connect (G_OBJECT (button), "clicked", G_CALLBACK (ouverture), (gpointer) param);
  gtk_box_pack_start (GTK_BOX (vbox), button, TRUE, TRUE, 0);

  button = gtk_button_new_with_label ("Ouvrir Groupe"); /*GTK_STOCK_DIALOG_INFO);*/
  sprintf(chemin,"%s%cgroupe.png",ici,SEPARATEUR);
  gtk_button_set_image(GTK_BUTTON(button),gtk_image_new_from_file(chemin));
  g_signal_connect (G_OBJECT (button), "clicked", G_CALLBACK (ouvrir_groupe), (gpointer) param);
  gtk_box_pack_start (GTK_BOX (vbox), button, TRUE, TRUE, 0);

  button = gtk_button_new_from_stock (GTK_STOCK_NEW); /*GTK_STOCK_DIALOG_INFO);*/
  g_signal_connect (G_OBJECT (button), "clicked", G_CALLBACK (nouveau), (gpointer) param);
  gtk_box_pack_start (GTK_BOX (vbox), button, TRUE, TRUE, 0);

  button = gtk_button_new_from_stock (GTK_STOCK_APPLY); /*GTK_STOCK_DIALOG_INFO);*/
  g_signal_connect (G_OBJECT (button), "clicked", G_CALLBACK (tirage), (gpointer) param);
  gtk_box_pack_start (GTK_BOX (vbox), button, TRUE, TRUE, 0);

  button = gtk_button_new_with_label ("Nouveaux Dieux"); /*GTK_STOCK_DIALOG_INFO);*/
  sprintf(chemin,"%s%cdieux.png",ici,SEPARATEUR);
  gtk_button_set_image(GTK_BUTTON(button),gtk_image_new_from_file(chemin));
  g_signal_connect (G_OBJECT (button), "clicked", G_CALLBACK (nouveau_dieux),win);
  gtk_box_pack_start (GTK_BOX (vbox), button, TRUE, TRUE, 0);

  button = gtk_button_new_from_stock (GTK_STOCK_CLOSE);
  g_signal_connect (button, "clicked", G_CALLBACK (cb_fin),(gpointer)  param);
  gtk_box_pack_start (GTK_BOX (vbox), button, TRUE, TRUE, 0);

  /* inactivation de certaines entrées du menu */
  non_humour_beaute(p_actionGroup);
  /*inactive("illusionniste",p_actionGroup);
  inactive("beaute",p_actionGroup);*/
  //inactive("demi_orc",p_actionGroup);
  //inactive("assassin",p_actionGroup);
  //inactive("moine",p_actionGroup);
  //inactive("autre",p_actionGroup);
  //inactive("r_autre",p_actionGroup);
  param[8]=(void *)gtk_action_group_list_actions (p_actionGroup);
  envp_init(param);
  race_init  (menu_Manager,param);
  classe_init(menu_Manager,param);

  /* ajoute les Tearoff */
  Tearoff_on(menu_Manager,"/menubar/Tirage/ModeDeTirage");
  Tearoff_on(menu_Manager,"/menubar/Tirage/Classes");
  Tearoff_on(menu_Manager,"/menubar/Tirage/race");
  Tearoff_on(menu_Manager,"/menubar/Tirage/sexe");
  Tearoff_on(menu_Manager,"/menubar/Tirage/preference");
  Tearoff_on(menu_Manager,"/menubar/Tirage/preference/force");
  Tearoff_on(menu_Manager,"/menubar/Tirage/preference/intelligence");
  Tearoff_on(menu_Manager,"/menubar/Tirage/preference/sagesse");
  Tearoff_on(menu_Manager,"/menubar/Tirage/preference/dexterite");
  Tearoff_on(menu_Manager,"/menubar/Tirage/preference/constitution");
  Tearoff_on(menu_Manager,"/menubar/Tirage/preference/charisme");
  Tearoff_on(menu_Manager,"/menubar/Tirage/preference/beaute");
  Tearoff_on(menu_Manager,"/menubar/Tirage/preference/humour");
  Tearoff_on(menu_Manager,"/menubar/Tirage/pdv");
  Tearoff_on(menu_Manager,"/menubar/Tirage/niveau");

  free(ici);
  g_signal_connect (win, "destroy", G_CALLBACK (cb_fin), (gpointer) param);
  //g_signal_connect (win, "show", G_CALLBACK (attente), (gpointer) param);


  gtk_widget_show_all (win);

  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(((struct_classe **)param)[MAIN_CLASSE][0].menu),0);
  #ifdef __linux__
  unique_app_watch_window (app, GTK_WINDOW (win));
  g_signal_connect (app, "message-received", G_CALLBACK (attente), param);
  #endif

  for (i=1;i<argc;i++)
  {
      ouvre_fichier(win,param,argv[i],0);
  }

/*{
GList * gl=gtk_action_group_list_actions (p_actionGroup),*k;
FILE * sortie;

sortie=fopen("menu.txt","w");

for (k = gl; k != NULL; k = g_list_next(k))
{
    gchar *action_label;
    gchar *action_name;
    gchar *action_stock;
    GtkAction *action;
    action = GTK_ACTION(k->data);
    g_object_get(action,"label",&action_label,"name",&action_name,"stock-id", &action_stock,NULL);
    fprintf(sortie,"label :%s , name : %s , stock : %s\n",action_label,action_name,action_stock);
    //if (strcmp(action_name,"_3D6")==0)
    {
         guint nb_s,f;
         GParamSpec ** l_spec=g_object_class_list_properties(G_OBJECT_GET_CLASS(action),&nb_s);
         for (f=0;f<nb_s;f++)
         {
             fprintf(sortie,"   %s de type %s\n",l_spec[f]->name,g_type_name(l_spec[f]->owner_type));
         }
    }
    g_free(action_stock);
    g_free(action_name);
    g_free(action_label);
}
fclose(sortie);
}*/

  gtk_main ();
  #ifdef __linux__
  g_object_unref (app);
  #endif
  return 0;
}

void ouvre_nouveau_fichier(GtkWidget *wid, gpointer param)
{
    ouvre_fichier(wid,param,(char *)(gtk_entry_get_text(((GtkEntry **)param)[7])),0);
    (void) wid;
    (void) param;
}

void ouverture (GtkWidget *wid, gpointer param)
{
    nouv_ouv(wid,param,OUVRIR);
}

void nouveau (GtkWidget *wid, gpointer param)
{
    nouv_ouv(wid,param,NOUVEAU);
}


#ifdef __linux__
static UniqueResponse attente (UniqueApp * app,UniqueCommand command,UniqueMessageData *message,guint time_,gpointer param)
{
   ouvre_fichier(((GtkWidget **)param)[0],param,unique_message_data_get_text(message),0);

   (void)app;
   (void)command;
   (void)time_;

   return UNIQUE_RESPONSE_OK;
}
#endif
